@@ -11,7 +11,8 @@ use strict;
 use Number::Range::Regex::Range;
 use Number::Range::Regex::Iterator;
 use Number::Range::Regex::Util;
-use vars qw ( @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION ); 
+use Number::Range::Regex::Util::inf qw( neg_inf pos_inf );
+use vars qw ( @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION );
 eval { require warnings; }; #it's ok if we can't load warnings
 
 require Exporter;
@@ -21,11 +22,9 @@ use base 'Exporter';
 @EXPORT_OK = qw ( init regex_range ) ;
 %EXPORT_TAGS = ( all => [ @EXPORT, @EXPORT_OK ] );
 
-$VERSION = '0.20';
+$VERSION = '0.32';
 
-my $init_opts = {};
-
-sub features { return { negative => 1 }; }
+my $init_opts = $Number::Range::Regex::Range::default_opts;
 
 sub init {
   my ($self, @opts) = @_;
@@ -46,87 +45,60 @@ sub init {
 #   regex_range(3, undef) yields the equivalent of qr/[+]?[3-9]|\d+/;
 sub regex_range {
   my ($min, $max, $passed_opts) = @_;
-
-  my $opts;
-  if($passed_opts) {
-    die "regex_range: too many arguments" unless ref $passed_opts eq 'HASH';
-    $opts = { %$init_opts };
-    while (my ($key, $val) = each %$passed_opts) {
-      $opts->{$key} = $val;
-    }
-  } else {
-    $opts = $init_opts;
-  }
-
-  return range($min, $max, $opts)->regex( $opts );
+  my $opts = option_mangler( $init_opts, $passed_opts );
+  return range($min, $max, $opts)->regex();
 }
 
 sub range {
-  my ($min, $max, $opts) = @_;
-  my $range;
-  undef $min  if  defined $min && $min eq '-inf';
-  undef $max  if  defined $max && $max =~ /^\+?inf$/;
-  if(defined $min) {
-    if(defined $max) {
-      return Number::Range::Regex::SimpleRange->new( $min, $max, $opts );
-    } else {
-      return Number::Range::Regex::InfiniteRange->new_positive_infinity( $min, $opts );
-    }
-  } else {
-    if(defined $max) {
-      return Number::Range::Regex::InfiniteRange->new_negative_infinity( $max, $opts );
-    } else {
-      die "must specify either a min or a max" if !$opts->{allow_wildcard};
-      return Number::Range::Regex::InfiniteRange->new_both( $opts );
-    }
+  my $opts = option_mangler( ref $_[-1] eq 'HASH' ? pop : undef );
+  my ($min, $max) = @_;
+  if(!defined $min && !defined $max) {
+    die "for the set of all integers, you must specify min as '-inf' and max as '+inf', or use the allow_wildcard argument" if !$opts->{allow_wildcard};
   }
+  $min = neg_inf  if  !defined $min;
+  $max = pos_inf  if  !defined $max;
+  return rangespec( "$min..$max", $opts );
 }
 
 sub rangespec {
-  my $opts;
-  if(ref $_[-1]) {
-    $opts = option_mangler( pop );
-  }
+  my $opts = option_mangler( ref $_[-1] eq 'HASH' ? pop : undef );
   # we allow (but do not like) e.g. rangespec(5,7,10..18);
   # we don't like it because it can make us run out of memory for
   # large ranges. preferred: rangespec('5,7,10..18');
-  my $range;
+  my $spec;
   if(@_ > 1) {
     warn "passed literal range to rangespec!\n";
-    $range = join ',', @_;
+    $spec = join $opts->{range_separator}, @_;
   } else {
-    $range = $_[0];
+    $spec = $_[0];
   }
 
-  my $section_validate  = qr/(?:-?\d+\.\.-?\d+|-?\d+|-?\d+\.\.\+?inf|-inf\.\.-?\d+)/;
-  my $range_validate = qr/(?:|$section_validate(?:,\s*$section_validate)*)/;
-  $range =~ s/\s+//g;
-  die "invalid rangespec '$range'"  unless  $range =~ /^$range_validate$/;
+  my $base = $opts->{base};
+  my $base_digits = base_digits($base);
+  my $base_max   = substr($base_digits, -1);
 
-  my @sections = split /,\s*/, $range;
+  my $digits_validate  = "[$base_digits]+";
+  my $range_operator   = '\s*'.quotemeta( $opts->{range_operator} ).'\s*';
+  my $range_separator  = '\s*'.quotemeta( $opts->{range_separator} ).'\s*';
+  my $section_validate = qr/(?:-?$digits_validate|(?:-?$digits_validate|-inf)$range_operator(?:\+?inf|-?$digits_validate))/;
+  my $range_validate   = qr/(?:|$section_validate(?:$range_separator$section_validate)*)/;
+  die "invalid rangespec '$spec' !~ /$range_validate/"  unless  $spec =~ /^$range_validate$/;
+
+  my @sections = split /$range_separator/, $spec;
   my @ranges;
   foreach my $section (@sections) {
-    if($section =~ /^(\d+)$/) {
+    if($section =~ /^(-?$digits_validate)$/) {
       push @ranges, Number::Range::Regex::SimpleRange->new( $1, $1, $opts );
     } else {
-      my ($min, $max) = split /\.\./, $section, 2;
-      undef $min  if  defined $min && $min eq '-inf';
-      undef $max  if  defined $max && $max =~ /^\+?inf$/;
-      if(defined $min && defined $max) {
-        push @ranges, Number::Range::Regex::SimpleRange->new( $min, $max, $opts );
-      } elsif(defined $min) {
-        push @ranges, Number::Range::Regex::InfiniteRange->new_positive_infinity( $min );
-      } elsif(defined $max) {
-        push @ranges, Number::Range::Regex::InfiniteRange->new_negative_infinity( $max );
-      } else { #min and max are undefined
-        push @ranges, Number::Range::Regex::InfiniteRange->new_both();
-      }
+      my ($min, $max) = map { s/^\s+//; s/\s+$//; $_ } split /$range_operator/, $section, 2;
+      push @ranges, Number::Range::Regex::SimpleRange->new( $min, $max, $opts );
     }
   }
+  my $warn_overlap = defined $opts->{warn_overlap} ?
+                     $opts->{warn_overlap} : 'rangespec';
   # note: multi_union() will have the side effect of sorting
   #       and de-overlap-ify-ing the input ranges
-  return Number::Range::Regex::EmptyRange->new()  if  !@ranges;
-  return multi_union( @ranges );
+  return multi_union( @ranges, { warn_overlap => $warn_overlap } );
 }
 
 1;
@@ -165,7 +137,7 @@ Number::Range::Regex - create regular expressions that check for
   if( $nonprimes_lt_10->contains( $something ) ) {
     print "something($something) is a nonprime less than 10";
   }
-  
+
   my $octet = range(0, 255);
   my $ip4_match = qr/^$octet\.$octet\.$octet\.$octet$/;
   my $range_96_to_127 = range(96, 127);
@@ -179,7 +151,7 @@ Number::Range::Regex - create regular expressions that check for
   do { print $it->fetch } while ($it->next);
   $it->last;
   do { print $it->fetch } while ($it->prev);
-  
+
 
 =head1 DESCRIPTION
 
@@ -244,7 +216,7 @@ it returns a false value. e.g.
   rangespec('7..9')->overlaps( rangespec('10..12') ) => false
   rangespec('7..9')->overlaps( rangespec('6..10') )  => true
   rangespec('7..9')->overlaps( rangespec('6..7') )   => true
-  
+
 
 =back
 
@@ -254,7 +226,7 @@ it returns a false value. e.g.
 
 =item to_string
 
-  $range->to_sting();
+  $range->to_string();
 
 Return a compact representation of the range suitable for consumption
 by a human, perl(1), or rangespec(). For example:
@@ -286,17 +258,51 @@ Number::Range::Regex between v0.12 and v0.13, is:
 
 =head2 OVERLOADING
 
-=over
-
 Please note that range objects are overloaded so that in regex
 context, $range will be equivalent to $range->regex(). This
 works in all versions of perl >= v5.6.0. When it is further
 possible to distinguish regex context from string context (as
 in overload v1.10 or higher, available in perl >= v5.12.0),
-range objects will display in string context as the terser
-$range->to_string() instead. 
+range objects will display in string but not regex context as
+the terser, more legible $range->to_string() instead.
 
-=back
+If you find any cryptic errors about overloading, please use
+an explicit ->to_string or ->regex() and file a bug.
+
+=head2 UNBOUNDED (aka "infinite") RANGES
+
+It is also possible to specify unbounded ranges, ie the set of
+all integers less than 17. This may be specified in any of the
+following ways:
+
+  range( undef, 17 );
+  range( '-inf', 17);
+  rangespec('-inf..17');
+
+Similarly the set of all integers greater than 17:
+
+  range( 17, undef );
+  range( 17, '+inf' );
+  rangespec('17..+inf');
+
+Note carefully that, in order to prevent errors, it is not possible to
+specify the set of all integers via range(undef, undef). If you try to
+do so, Number::Range::Regex will complain that you "must specify either
+a min or a max or use the allow_wildcard argument":
+
+  range( undef, undef );   #boom
+
+If you really want range() with no defined arguments to mean the set of
+all possible integers, you can use one of the below:
+
+  range( undef, undef, {allow_wildcard => 1} ) ;
+  range( '-inf', '+inf' );
+
+To test if a range is infinite, you can call is_infinite():
+
+  1 == ! rangespec('3..7')->is_infinite();
+  1 == rangespec('16..+inf')->is_infinite();
+  1 == rangespec('')->not->is_infinite();
 
 =head2 SET OPERATIONS
 
@@ -326,8 +332,9 @@ via the alias intersection.
 
   $range = $range->xor( $another_range );
 
-Return the symmetric difference of $range2 and $range3. In the example
-above, $range would consist of 2, 3, 4, 8, and 9.
+Return the symmetric difference of $range2 and $range3 (elements in one
+or the other range, but not in both). In the example above, $range would
+consist of 2, 3, 4, 8, and 9.
 
 =item subtract
 
@@ -344,19 +351,22 @@ available via the aliases subtraction and minus.
   $range = $range2->invert();
 
 Return the absolute complement of $range2. In the example above,
-$range would consist of: 
-  any integer less than or equal to -1
-  1, 3, 5, and 7,  and
-  any integer greater than or equal to 9
-This method is also available via the alias not.
-
-=back
-
-=head2 OTHER METHODS
+$range would include:
 
 =over
 
-=item iterator
+  any integer less than or equal to -1
+  1, 3, 5, and 7,  and
+  any integer greater than or equal to 9.
+
+=back
+
+that is, $range->to_string would be '-inf..-1,1,3,5,7,9..+inf'. This
+method is also available via the alias not.
+
+=back
+
+=head2 ITERATORS
 
 iterators let you examine large or infinite sets with minimal memory:
 
@@ -368,14 +378,19 @@ iterators let you examine large or infinite sets with minimal memory:
 
 for more information, see Number::Range::Regex::Iterator
 
-=item regex_range [ DEPRECATED ]
+=head2 OTHER METHODS
+
+=over
+
+=item regex_range
 
   $regex = regex_range( $min, $max );
 
 This is a shortcut for range( $min, $max )->regex(). Useful for
-one-off use when overload.pm does not support regex context.
-This method is deprecated, and may disappear with the next
-release of Number::Range::Regex. This method is not exported
+one-off use when overload.pm does not support regex context. It
+should only be used with perl 5.10.X or lower where regex context
+overloading is not possible. This method may be deprecated in a
+future release of Number::Range::Regex. This method is not exported
 by default.
 
 =back
